/*
**  MethodViewer.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MethodViewer.h"
#include "Utilities.h"
#include <AppKit/AppKit.h>
#include <CodeEditorView/CodeEditorView.h>
#include <CodeEditorView/RulesetManager.h>
#include <CodeEditorView/RegEx.h>

static MethodViewer *sharedViewer;

@implementation MethodViewer

+ (MethodViewer *) sharedMethodViewer
{
  if (!sharedViewer)
    {
      sharedViewer = [[MethodViewer alloc] init];
    }
  return sharedViewer;
}

- (void) tableViewSelectionDidChange: (NSNotification *) not
{
  int index = [tableView selectedRow];
  unsigned position = [[positions objectAtIndex: index] unsignedIntValue];
  [currentCEView setSelectedRange: NSMakeRange(position, 0)];
}

- (int) numberOfRowsInTableView: (NSTableView *) tableView
{
  if (methods)
    return [methods count];
  return 0;
}

- (id) tableView: (NSTableView *) tableView
       objectValueForTableColumn: (NSTableColumn *) tableColumn
       row: (int) row
{
  if (methods)
    return [methods objectAtIndex: row];
  return @"";
}

- (void) windowDidBecomeKey: (NSNotification *) not
{
  [self setCodeEditorView: currentCEView];
}

- (void) setCodeEditorView: (CodeEditorView *) ceView
{
  Class regexParser = RegExParserClass();
  NSRange range;
  NSMutableArray *array, *array1;
  NSDictionary *dict = nil;
  NSEnumerator *e;
  NSMutableString *string;
  id object;

  ASSIGN(currentCEView, ceView);

  if (currentCEView == nil)
    return;
  
//  dict = [[RulesetManagerClass() sharedRulesetManager] fontificationRulesetWithName: [currentCEView languageName]];
  if (dict == nil)
    return;

  e = [[dict objectForKey: @"Tags"] objectEnumerator];
  if (e == nil)
    return;

  array = [NSMutableArray new];
  array1 = [NSMutableArray new];
  
  while ((object = [e nextObject]))
    {
      range = [regexParser rangeOfString: object
                           inString: [currentCEView string]];

      while (range.location != NSNotFound)
        {
          [array1 addObject: [NSNumber numberWithUnsignedInt: range.location]];
          string = [[NSMutableString alloc] initWithString: [[currentCEView string] substringFromRange: range]];

          [string replaceOccurrencesOfString: @"\n"
                                  withString: @""
                                     options: 0
                                       range: NSMakeRange(0, [string length])];
          [string replaceOccurrencesOfString: @"\r"
                                  withString: @""
                                     options: 0
                                       range: NSMakeRange(0, [string length])];
          [string replaceOccurrencesOfString: @"\t"
                                  withString: @""
                                     options: 0
                                       range: NSMakeRange(0, [string length])];
          [string replaceOccurrencesOfString: @"  "
                                  withString: @" "
                                     options: 0
                                       range: NSMakeRange(0, [string length])];

          [array addObject: string];
          range.location = NSMaxRange(range);
          range.length = [[currentCEView string] length] - range.location;
          range = [regexParser rangeOfString: object
                                   inString: [currentCEView string]
                                      range: range];
          RELEASE(string);
        }
    }
  ASSIGN(methods, [NSArray arrayWithArray: array]);
  ASSIGN(positions, [NSArray arrayWithArray: array1]);
  RELEASE(array);
  RELEASE(array1);

  [tableView reloadData];
}

- (NSPanel *) methodPanel
{
  if (methodPanel == nil)
    {
      if(NO == [NSBundle loadNibNamed: @"MethodPanel.gorm" owner: self])
        {
           NSBeep();
           return nil;
        }
    }
  [tableView setAutoresizesAllColumnsToFit: YES];
  [tableView setAllowsColumnSelection: NO];
  [tableView setAllowsColumnReordering: NO];
  [[[tableView tableColumnWithIdentifier: @"method"] headerCell] setStringValue: @"Methods"];
  [[tableView tableColumnWithIdentifier: @"method"] setMinWidth: 1000];

  return methodPanel;
}

- (void) orderFrontMethodPanel: (id) sender
{
  if ([self methodPanel] == nil)
    {
       NSBeep();
       return;
    }
  [methodPanel makeKeyAndOrderFront: self];
}

- (void) dealloc
{
  RELEASE(currentCEView);
  RELEASE(methods);
  RELEASE(positions);
  [super dealloc];
}

@end
