/*
**  ObjCHandler.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ObjCHandler.h"
#include <CodeEditorView/RulesetManager.h>
#include <AppKit/AppKit.h>

@implementation ObjCHandler

+ (NSString *) language
{
  return @"Objective-C";
}

+ (Class) blockHandlerClass
{
  return NSClassFromString(@"ObjCBlockHandler");
}

+ (Class) indentationHandlerClass
{
  return NSClassFromString(@"ObjCIndentHandler");
}

- (void) string: (NSString *) element
{
  NSAttributedString *as = [[NSAttributedString alloc] initWithString: element];
  NSColor *color = [NSColor blackColor];
  NSFont *font;
  unsigned int len = [element length];
  NSRange attributeRange = NSMakeRange([_result length], len);

  [super string: element];

  font = [_rulesetManager convertFont: nil toType: @"Normal"];

  /* Comments */
  if (_commentType != NoComment)
    {
      color = [_rulesetManager colorForType: @"Comments"];
      attributeRange = NSMakeRange([_result length], len);
      font = [_rulesetManager convertFont: nil toType: @"Comments"];
    }
  else if (_stringBegin == YES)
    {
      color = [_rulesetManager colorForType: @"Strings"];
      attributeRange = NSMakeRange([_result length], len);
      font = [_rulesetManager convertFont: nil toType: @"Strings"];
    }
  /* Keyword */
  else if ( EQUAL(@"break") || EQUAL(@"extern") || EQUAL(@"continue") || EQUAL(@"_cmd") ||
            EQUAL(@"self") || EQUAL(@"super") || EQUAL(@"return") || EQUAL(@"sizeof") ||
            EQUAL(@"break") || EQUAL(@"case") || EQUAL(@"default") ||
            EQUAL(@"goto") || EQUAL(@"switch") || EQUAL(@"if") || EQUAL(@"else") ||
            EQUAL(@"do") || EQUAL(@"shile") || EQUAL(@"for") || EQUAL(@"while")
          )
    {
      color = [_rulesetManager colorForType: @"Keywords"];
      attributeRange = NSMakeRange([_result length], len);
      font = [_rulesetManager convertFont: nil toType: @"Keywords"];
    }
  else if ( [_preString isEqualToString: @"@"] &&
            ( EQUAL(@"class") || EQUAL(@"selector") || EQUAL(@"interface") ||
              EQUAL(@"implementation") || EQUAL(@"end") || EQUAL(@"encode") ||
              EQUAL(@"private") || EQUAL(@"protected") || EQUAL(@"char")
            )
          )
    {
      color = [_rulesetManager colorForType: @"Keywords"];
      attributeRange = NSMakeRange([_result length]-1, len+1);
      font = [_rulesetManager convertFont: nil toType: @"Keywords"];
    }

  /* Preprocessor */
  else if ( [_preString isEqualToString: @"#"] &&
            ( EQUAL(@"import") || EQUAL(@"include") || EQUAL(@"ifdef") ||
              EQUAL(@"ifndef") || EQUAL(@"if defined") || EQUAL(@"else") ||
              EQUAL(@"endif") || EQUAL(@"pragma") || EQUAL(@"define") ||
              EQUAL(@"warning") || EQUAL(@"error")
            )
          )
    {
      color = [_rulesetManager colorForType: @"Preprocessor"];
      attributeRange = NSMakeRange([_result length]-1, len+1);
      font = [_rulesetManager convertFont: nil toType: @"Preprocessor"];
    }

  /* KnownType */
  else if ( EQUAL(@"int") || EQUAL(@"long") || EQUAL(@"short") || EQUAL(@"char") ||
            EQUAL(@"float") || EQUAL(@"double") || EQUAL(@"void") || EQUAL(@"union") ||
            EQUAL(@"unichar") || EQUAL(@"const") || EQUAL(@"signed") || 
            EQUAL(@"unsigned") || EQUAL(@"static") || EQUAL(@"volatile") ||
            EQUAL(@"enum") || EQUAL(@"id") || EQUAL(@"SEL") || EQUAL(@"BOOL") ||
            EQUAL(@"Class") || EQUAL(@"Nil") || EQUAL(@"IBAction") ||
            EQUAL(@"IBOutlet") ||
            [element hasPrefix: @"NS"] )
    {
      color = [_rulesetManager colorForType: @"KnownTypes"];
      attributeRange = NSMakeRange([_result length], len);
      font = [_rulesetManager convertFont: nil toType: @"KnownTypes"];
    }

  /* Strings */
  else if ( EQUAL(@"FALSE") || EQUAL(@"NO") || EQUAL(@"TRUE") ||
            EQUAL(@"YES") || EQUAL(@"nil") )
    {
      color = [_rulesetManager colorForType: @"Strings"];
      attributeRange = NSMakeRange([_result length], len);
      font = [_rulesetManager convertFont: nil toType: @"Strings"];
    }
 
  [_result appendAttributedString: as];
  attributeRange.location += _startRange.location;
/*
  [_result addAttribute: NSForegroundColorAttributeName
                  value: color
                  range: attributeRange];
  [_result addAttribute: NSFontAttributeName
                  value: font
                  range: attributeRange];
*/
  [_origin addAttribute: NSForegroundColorAttributeName
                  value: color
                  range: attributeRange];
  [_origin addAttribute: NSFontAttributeName
                  value: font
                  range: attributeRange];
  ASSIGN(_preString, element);
}

- (void) number: (NSString *) element 
{
  NSAttributedString *as;

  [super number: element];

  as = [[NSAttributedString alloc] initWithString: element];
  [_result appendAttributedString: as];
  RELEASE(as);
  ASSIGN(_preString, element);
}

- (void) spaceAndNewLine: (NSString *) element 
{
  NSAttributedString *as;

  [super spaceAndNewLine: element];

  as = [[NSAttributedString alloc] initWithString: element];
  [_result appendAttributedString: as];
  RELEASE(as);
  ASSIGN(_preString, element);
}

- (void) symbol: (NSString *) element 
{
  NSAttributedString *as = [[NSAttributedString alloc] initWithString: element];
  NSColor *color = [NSColor blackColor];
  NSRange attributeRange;
  unsigned int len = [element length];
  NSString *temp;

  [super symbol: element];

  temp = [[NSString alloc] initWithFormat: @"%@%@", _preString, element];

  if ( [temp isEqualToString: @"/*"] ||
       [temp isEqualToString: @"//"] ||
       [temp isEqualToString: @"*/"] )
    {
      color = [_rulesetManager colorForType: @"Comments"];
      attributeRange = NSMakeRange([_result length]-1, len+1);
      attributeRange.location += _startRange.location;
      [_result appendAttributedString: as];
/*
      [_result addAttribute: NSForegroundColorAttributeName
                      value: color
                      range: attributeRange];
*/
      [_origin addAttribute: NSForegroundColorAttributeName
                      value: color
                      range: attributeRange];
    }
  else
    {
      if (_commentType != NoComment)
        {
          color = [_rulesetManager colorForType: @"Comments"];
          attributeRange = NSMakeRange([_result length], len);
          attributeRange.location += _startRange.location;
          [_result appendAttributedString: as];
/*
          [_result addAttribute: NSForegroundColorAttributeName
                          value: color
                          range: attributeRange];
*/
          [_origin addAttribute: NSForegroundColorAttributeName
                          value: color
                          range: attributeRange];
        }
      else
        {
          [_result appendAttributedString: as];
        }
    }
  ASSIGN(_preString, element);
}

- (void) invisible: (NSString *) element
{
  NSAttributedString *as;

  [super invisible: element];

  as = [[NSAttributedString alloc] initWithString: element];
  [_result appendAttributedString: as];
  RELEASE(as);
  ASSIGN(_preString, element);
}

- (void) setString: (NSMutableAttributedString *) string
{
  ASSIGN(_origin, string);
  _startRange = NSMakeRange(0, [_origin length]);
}

- (void) setRange: (NSRange) range
{
  _startRange = range;
}

- (id) result
{
  return _result;
}

- (id) init
{
  self = [super init];
  _result = [[NSMutableAttributedString alloc] init];
  _rulesetManager = [RulesetManager sharedRulesetManager];
  _startRange = NSMakeRange(0, 0); 
  return self;
}

- (void) dealloc
{
  RELEASE(_origin);
  RELEASE(_result);
  [super dealloc];
}

@end

